==> machi-auth-1.0.62.tgz: package/lib/middleware.js <==
﻿var moment = require('moment-timezone');
var accessTokens = require('./accessTokens');
var url = require('url');

module.exports = function (app) {

    var machiModule = app.get('machiModule');
    var options = machiModule.options;

    var middleWareObj = {};

    var uiLoginGroups = ['challenge 1', 'Machina Internal', 'Upworkers'];
    var employeesLoginGroups = ['Machina Internal'];
    var adminsLoginGroups = ['admins'];
    var ensureLoggedIn = require('connect-ensure-login').ensureLoggedIn();
    /**
     * Make sure the user is existing
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    //middleWareObj.loginRequired = require('connect-ensure-login').ensureLoggedIn();
    middleWareObj.loginRequired = function(req, res, next) {
        if (!req.user)
            return ensureLoggedIn(req, res, next);

        machiModule.connect.getUser(req.user.username, (err, user) => {
            if (err || user.status != 'active')
                machiModule.connect.logout(req, res, false);
            ensureLoggedIn(req, res, next);
        });
    }
    /**
     * If request has valid accessToken then Authorize the request
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.accessTokenAuthorize = function (req, res, next) {
        if (req.user) return next();

        var tokenInfo = accessTokens.verifyJwtToken(req);
        if (tokenInfo)
            req.user = tokenInfo.user;
        next();
    };

    /**
     * If request has basic auth then Authorize the request
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.basicAuthorize = function (req, res, next) {
        if (req.user) return next();

        var authHeader = req.headers.authorization;
        if (!authHeader)
            return next();
        var auth = new Buffer(authHeader.substring(6), 'base64').toString().split(':');
        machiModule.connect.validateLogin(auth[0], auth[1], function (err, user) {
            if (err) return next();
            req.user = user;
            var token = accessTokens.createJwtToken(req, {}, 60 * 60).accessToken;
            res.header('accessToken', token);
			res.header('access_token', token);
            next();
        });
    };

    /**
     * Make sure the user is belong to admin group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.adminLoginRequired = function (req, res, next) {
        middleWareObj.groupsRequired(adminsLoginGroups)(req, res, next);
    };

    /**
     * Make sure the user is belong to employees group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.employeesLoginRequired = function (req, res, next) {
        middleWareObj.groupsRequired(employeesLoginGroups)(req, res, next);
    };


    /**
     * Make sure the user is belong to authorized UI group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.uiLoginRequired = function (req, res, next) {
        middleWareObj.groupsRequired(uiLoginGroups)(req, res, next);
    };


    /**
     * Make sure the user is belong to authorized group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.groupsRequired = function (groups) {
        return function (req, res, next) {
            middleWareObj.loginRequired(req, res, function () {
                middleWareObj.isGroupMembership(req.user.username, groups, function (err, isInGroup) {
                    if (err || isInGroup) return next(err);
                    res.redirect('/unauthorized');
                });
            });
        }
    };
    /**
     * read the user object from authorization header or from cookies and reply with bad auth user/pass in case wrong log in
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.apiAuthenticationRequired = apiMiddleware(false);

    /**
     * read the user object from authorization header or from cookies and reply with bad auth user/pass in case wrong log in
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.apiAdminAuthenticationRequired = apiMiddleware(true);


     /**
     * Make sure all sockets must have true admin username
     *
     * @method
     *
     * @param {socket.io} socket - The socket.io object.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.socketAuthenticationRequired = socketMiddleware(false);
    middleWareObj.socketAdminAuthenticationRequired = socketMiddleware(true);

    function socketMiddleware(adminRequired) {
        return function (socket, next) {
            socket.socketClientIP = socket.handshake.headers['x-forwarded-for'] || socket.handshake.address;
            socket.dateTime = moment.utc().format('YYYYMMDDTHHmm');
            var req = socket.conn.request;
            machiModule.cookieMiddleware(req, {}, function () {
                req.user = socket.user = req.session && req.session.passport && req.session.passport.user;
                
                if (!socket.user)
                    next('Not Authorized');
                else if (adminRequired)
                    middleWareObj.isAdminLogin(req, next);
                else next();
            });
        }
    }
    /**
     * Checks if req has a valid session 
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    function apiMiddleware(adminRequired) {

        return function (req, res, next) {
            if (!req.user)
                return badAuth(req, res);
            if (!adminRequired) return next();
            middleWareObj.isAdminLogin(req, function (err, isAdmin) {
                if (err || !isAdmin)
                    return badAuth(req, res);
                return next();
            });
        }
    }

    //reply with bad auth header user/pass
    function badAuth(req, res) {
        res.statusCode = 401;
        // res.setHeader('WWW-Authenticate', 'Basic realm=""');
        res.end('Unauthorized');
    }
    /**
     * Make sure the user is belong to admin group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isAdminLogin = function (req, next) {
        middleWareObj.isGroupMembership(req.user.username, adminsLoginGroups, next);
    };


    /**
     * Make sure the user is belong to ui groups
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isUILogin = function (req, next) {
        middleWareObj.isGroupMembership(req.user.username, uiLoginGroups, next);
    };

    /**
     * Make sure the user is belong to employees group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isEmployeesLogin = function (req, next) {
        middleWareObj.isGroupMembership(req.user.username, employeesLoginGroups, next);
    };




    /**
     * Make sure the user is belong to one of the groups
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isGroupMembership = function (username, groupsNames, next) {

        machiModule.connect.getUserGroups(username, function (err, groups) {
            if (err) return next(err);
            var isMemeber = groups.some(function (g) {
                return groupsNames.indexOf(g.name) >= 0;
            });
            next(undefined, isMemeber);
        });
    };

    /**
     * Allow invoke the api from cross origin
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.allowOriginHeaders = function (req, res, next) {
        var origin = url.parse(req.headers.origin || '').hostname;
        if (origin && (process.env.NODE_ENV !== 'production'|| origin.endsWith('.machi.na')|| origin.endsWith('machina.trydiscourse.com'))) {
            // Website you wish to allow to connect
            res.setHeader('Access-Control-Allow-Origin', req.headers.origin);

            // Request methods you wish to allow
            res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');

            // Request headers you wish to allow
            res.setHeader('Access-Control-Allow-Headers', 'X-Requested-With,content-type,*');

            // Set to true if you need the website to include cookies in the requests sent
            // to the API (e.g. in case you use sessions)
            res.setHeader('Access-Control-Allow-Credentials', true);
        }
        // Pass to next layer of middleware
        next();
    }


    return middleWareObj;
}

==> machi-auth-1.0.66.tgz: package/lib/middleware.js <==
﻿var moment = require('moment-timezone');
var accessTokens = require('./accessTokens');
var url = require('url');

module.exports = function (app) {

    var machiModule = app.get('machiModule');
    var options = machiModule.options;

    var middleWareObj = {};

    var uiLoginGroups = ['challenge 1', 'Machina Internal', 'Upworkers'];
    var employeesLoginGroups = ['Machina Internal'];
    var adminsLoginGroups = ['admins'];
    var ensureLoggedIn = require('connect-ensure-login').ensureLoggedIn();
    /**
     * Make sure the user is existing
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    //middleWareObj.loginRequired = require('connect-ensure-login').ensureLoggedIn();
    middleWareObj.loginRequired = function(req, res, next) {
        if (!req.user)
            return ensureLoggedIn(req, res, next);

        machiModule.connect.getUser(req.user.username, (err, user) => {
            if (err || user.status != 'active')
                machiModule.connect.logout(req, res, false);
            ensureLoggedIn(req, res, next);
        });
    }
    /**
     * If request has valid accessToken then Authorize the request
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.accessTokenAuthorize = function (req, res, next) {
        if (req.user) return next();

        var tokenInfo = accessTokens.verifyJwtToken(req);
        if (tokenInfo)
            req.user = tokenInfo.user;
        next();
    };

    /**
     * If request has basic auth then Authorize the request
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.basicAuthorize = function (req, res, next) {
        if (req.user) return next();

        var authHeader = req.headers.authorization;
        if (!authHeader)
            return next();
        var auth = new Buffer(authHeader.substring(6), 'base64').toString().split(':');
        machiModule.connect.validateLogin(auth[0], auth[1], function (err, user) {
            if (err) return next();
            req.user = user;
            var token = accessTokens.createJwtToken(req, {}, 60 * 60).accessToken;
            res.header('accessToken', token);
			res.header('access_token', token);
            next();
        });
    };

    /**
     * Make sure the user is belong to admin group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.adminLoginRequired = function (req, res, next) {
        middleWareObj.groupsRequired(adminsLoginGroups)(req, res, next);
    };

    /**
     * Make sure the user is belong to employees group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.employeesLoginRequired = function (req, res, next) {
        middleWareObj.groupsRequired(employeesLoginGroups)(req, res, next);
    };


    /**
     * Make sure the user is belong to authorized UI group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.uiLoginRequired = function (req, res, next) {
        middleWareObj.groupsRequired(uiLoginGroups)(req, res, next);
    };


    /**
     * Make sure the user is belong to authorized group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.groupsRequired = function (groups) {
        return function (req, res, next) {
            middleWareObj.loginRequired(req, res, function () {
                middleWareObj.isGroupMembership(req.user.username, groups, function (err, isInGroup) {
                    if (err || isInGroup) return next(err);
                    res.redirect('/unauthorized');
                });
            });
        }
    };
    /**
     * read the user object from authorization header or from cookies and reply with bad auth user/pass in case wrong log in
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.apiAuthenticationRequired = apiMiddleware(false);

    /**
     * read the user object from authorization header or from cookies and reply with bad auth user/pass in case wrong log in
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.apiAdminAuthenticationRequired = apiMiddleware(true);


     /**
     * Make sure all sockets must have true admin username
     *
     * @method
     *
     * @param {socket.io} socket - The socket.io object.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.socketAuthenticationRequired = socketMiddleware(false);
    middleWareObj.socketAdminAuthenticationRequired = socketMiddleware(true);

    function socketMiddleware(adminRequired) {
        return function (socket, next) {
            socket.socketClientIP = socket.handshake.headers['x-forwarded-for'] || socket.handshake.address;
            socket.dateTime = moment.utc().format('YYYYMMDDTHHmm');
            var req = socket.conn.request;
            machiModule.cookieMiddleware(req, {}, function () {
                req.user = socket.user = req.session && req.session.passport && req.session.passport.user;
                
                if (!socket.user)
                    next('Not Authorized');
                else if (adminRequired)
                    middleWareObj.isAdminLogin(req, next);
                else next();
            });
        }
    }
    /**
     * Checks if req has a valid session 
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {Object} res - The http response.
     * @param {function} next - A callback funtion for next function
     */
    function apiMiddleware(adminRequired) {

        return function (req, res, next) {
            if (!req.user)
                return badAuth(req, res);
            if (!adminRequired) return next();
            middleWareObj.isAdminLogin(req, function (err, isAdmin) {
                if (err || !isAdmin)
                    return badAuth(req, res);
                return next();
            });
        }
    }

    //reply with bad auth header user/pass
    function badAuth(req, res) {
        res.statusCode = 401;
        // res.setHeader('WWW-Authenticate', 'Basic realm=""');
        res.end('Unauthorized');
    }
    /**
     * Make sure the user is belong to admin group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isAdminLogin = function (req, next) {
        middleWareObj.isGroupMembership(req.user.username, adminsLoginGroups, next);
    };


    /**
     * Make sure the user is belong to ui groups
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isUILogin = function (req, next) {
        middleWareObj.isGroupMembership(req.user.username, uiLoginGroups, next);
    };

    /**
     * Make sure the user is belong to employees group
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isEmployeesLogin = function (req, next) {
        middleWareObj.isGroupMembership(req.user.username, employeesLoginGroups, next);
    };




    /**
     * Make sure the user is belong to one of the groups
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.isGroupMembership = function (username, groupsNames, next) {

        machiModule.connect.getUserGroups(username, function (err, groups) {
            if (err) return next(err);
            var isMemeber = groups.some(function (g) {
                return groupsNames.indexOf(g.name) >= 0;
            });
            next(undefined, isMemeber);
        });
    };

    /**
     * Allow invoke the api from cross origin
     *
     * @method
     *
     * @param {Object} req - The http request.
     * @param {function} next - A callback funtion for next function
     */
    middleWareObj.allowOriginHeaders = function (req, res, next) {
        var origin = url.parse(req.headers.origin || '').hostname;
        if (origin && (process.env.NODE_ENV !== 'production' || origin.endsWith('.equeum.com') || origin.endsWith('machina.trydiscourse.com'))) {
            // Website you wish to allow to connect
            res.setHeader('Access-Control-Allow-Origin', req.headers.origin);

            // Request methods you wish to allow
            res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');

            // Request headers you wish to allow
            res.setHeader('Access-Control-Allow-Headers', 'X-Requested-With,content-type,*');

            // Set to true if you need the website to include cookies in the requests sent
            // to the API (e.g. in case you use sessions)
            res.setHeader('Access-Control-Allow-Credentials', true);
        }
        // Pass to next layer of middleware
        next();
    }


    return middleWareObj;
}
